<template>
  <div class="h-full">
    <div
      class="border-b border-gray-200 p-6 flex justify-start items-center mb-4"
    >
      <el-button
        type="primary"
        :disabled="!form.projectId"
        @click="handleExport"
        >导出</el-button
      >
    </div>
    <div class="p-16">
      <el-form
        :model="form"
        :inline="true"
        :rules="queryFormRules"
        class="grid grid-cols-4"
      >
        <el-form-item label="项目名称" prop="projectId">
          <el-select v-model="form.projectId" @change="getTeamList" clearable>
            <el-option
              v-for="project in projectList"
              :key="project.id"
              :value="project.id"
              :label="project.itemProjectName"
              filter
              placeholder="请选择项目"
            ></el-option>
          </el-select>
        </el-form-item>
        <el-form-item label="团队组织名称">
          <el-select
            v-model="form.teamIdList"
            multiple
            :disabled="!form.projectId"
            clearable
            @change="getSiteList"
          >
            <el-option
              v-for="team in teamList"
              :key="team.id"
              :label="team.itemTeamName"
              :value="team.itemTeamId"
            ></el-option>
          </el-select>
        </el-form-item>
        <el-form-item label="日期">
          <el-date-picker
            v-model="dateRange"
            type="daterange"
            range-separator="至"
            start-placeholder="开始日期"
            end-placeholder="结束日期"
            value-format="yyyy-MM-dd"
            @change="handleDateChange"
          />
        </el-form-item>
        <el-form-item label="岗点名称">
          <!--          <el-input v-model="form.postName" placeholder="请输入" />-->
          <el-select
            v-model="form.siteIdList"
            multiple
            :disabled="
              !form.projectId ||
              !form.teamIdList ||
              form.teamIdList?.length <= 0
            "
            clearable
          >
            <el-option
              v-for="site in siteList"
              :key="site.id"
              :label="site.itemAttendanceSiteName"
              :value="site.id"
            ></el-option>
          </el-select>
        </el-form-item>
        <el-form-item label="姓名">
          <el-input v-model="form.userName" placeholder="请输入" />
        </el-form-item>
        <el-form-item>
          <el-button
            type="primary"
            @click="handleQuery"
            class="mr-2"
            :disabled="!form.projectId"
            >查询</el-button
          >
          <el-button @click="handleReset">重置</el-button>
        </el-form-item>
      </el-form>

      <el-table
        :data="tableData"
        :span-method="objectSpanMethod"
        border
        style="width: 100%"
      >
        <el-table-column prop="projectName" label="项目名称" />
        <el-table-column prop="statisticsDate" label="日期" />
        <el-table-column prop="teamName" label="团队组织名称" />
        <el-table-column prop="normalPeopleNumber" label="正常人数" />
        <el-table-column prop="abnormalPeopleNumber" label="异常人数" />
        <el-table-column prop="latePeopleNumber" label="迟到人数" />
        <el-table-column prop="leaveEarlyPeopleNumber" label="早退人数" />
        <el-table-column prop="groupName" label="考勤组" />
        <el-table-column prop="userName" label="姓名" />
        <el-table-column prop="shiftName" label="班次" />
        <el-table-column prop="shouldClockInCount" label="应打卡次数" />
        <el-table-column prop="actualClockInCount" label="实际打卡次数" />
        <el-table-column prop="lateCount" label="迟到次数" />
        <el-table-column prop="leaveEarlyCount" label="早退次数" />
        <el-table-column prop="missingCount" label="缺卡次数" />
        <el-table-column
          prop="shouldAttendanceTime"
          label="应出勤时长（小时）"
        />
        <el-table-column
          prop="actualAttendanceTime"
          label="实际出勤时长（小时）"
        />
        <el-table-column prop="workOvertime" label="加班时长（小时）" />
        <el-table-column prop="leaveTime" label="请假时长（小时）" />
      </el-table>

      <el-pagination
        :page-size="10"
        :total="total"
        :current-page="form.pageNum"
        layout="total, sizes, prev, pager, next, jumper"
        @size-change="onSizeChange"
        @current-change="handlePageChange"
        class="mt-4"
      />
    </div>
  </div>
</template>

<script>
import {
  getProject,
  getTeam,
  getSite,
  attendanceList,
  attendanceExport,
} from "@/api/statistics";

export default {
  name: "Duty",
  data() {
    return {
      dateRange: [],
      form: {
        projectId: "",
        teamIdList: [],
        statisticsEndTime: "",
        statisticsStartTime: "",
        siteIdList: [],
        userName: "",
        operateType: undefined,
        pageNum: 1,
        pageSize: 10,
      },
      queryFormRules: {
        projectId: [
          {
            required: true,
            message: "请选择项目",
            trigger: "change",
          },
        ],
      },
      projectList: [],
      teamList: [],
      siteList: [],
      tableData: [],
      total: 0,
    };
  },
  methods: {
    handleDateChange(val) {
      if (!val) {
        this.form.statisticsStartTime = "";
        this.form.statisticsEndTime = "";
      } else {
        this.form.statisticsStartTime = val[0];
        this.form.statisticsEndTime = val[1];
      }
    },
    getProjectList() {
      getProject().then((res) => {
        if (res) {
          this.projectList = res.data;
        }
      });
    },
    getTeamList() {
      this.form.teamIdList = [];
      if (!this.form.projectId) return;
      getTeam({
        projectId: this.form.projectId,
      }).then((res) => {
        if (res) {
          this.teamList = res;
        }
      });
    },
    getSiteList() {
      this.form.siteIdList = [];
      if (
        !this.form.projectId ||
        !this.form.teamIdList ||
        this.form.teamIdList?.length <= 0
      )
        return;
      getSite({
        projectId: this.form.projectId,
        teamIdList: this.form.teamIdList,
      }).then((res) => {
        if (res) {
          this.siteList = res;
        }
      });
    },
    generateSpans(data) {
      const spans = data.map(() => Array(9).fill({ rowspan: 1, colspan: 1 }));

      for (let col = 0; col < 9; col++) {
        let startRow = 0;
        let prevKeys = [];

        data.forEach((row, index) => {
          const currentKeys = this.getKeys(row, col);

          if (index === 0) {
            prevKeys = currentKeys;
            return;
          }

          // 检查是否需要合并
          if (!this.isSameKeys(prevKeys, currentKeys, col)) {
            const spanCount = index - startRow;
            spans[startRow][col] = { rowspan: spanCount, colspan: 1 };
            for (let i = startRow + 1; i < index; i++) {
              spans[i][col] = { rowspan: 0, colspan: 0 };
            }
            startRow = index;
            prevKeys = currentKeys;
          }

          // 处理最后一行
          if (index === data.length - 1) {
            const spanCount = index - startRow + 1;
            spans[startRow][col] = { rowspan: spanCount, colspan: 1 };
            for (let i = startRow + 1; i <= index; i++) {
              spans[i][col] = { rowspan: 0, colspan: 0 };
            }
          }
        });
      }
      return spans;
    },
    getKeys(row, column) {
      return [
        row.projectName,
        row.statisticsDate,
        row.teamName,
        row.normalPeopleNumber,
        row.abnormalPeopleNumber,
        row.latePeopleNumber,
        row.leaveEarlyPeopleNumber,
        row.groupName,
        row.userName,
      ].slice(0, column + 1);
    },
    isSameKeys(prev, current, depth) {
      for (let i = 0; i <= depth; i++) {
        if (prev[i] !== current[i]) return false;
      }
      return true;
    },
    objectSpanMethod({ rowIndex, columnIndex }) {
      if (columnIndex < 9) {
        // 生成合并配置
        const spans = this.generateSpans(this.tableData);
        return spans[rowIndex][columnIndex];
      }
      return { rowspan: 1, colspan: 1 };
    },
    handleQuery() {
      console.log("执行查询", this.form);
      attendanceList(this.form).then((res) => {
        console.log(res);
        if (res.code === 200) {
          this.tableData = res.rows || [];
          this.total = res.total || 0;
        }
      });
    },
    handleReset() {
      this.form = {
        projectId: "",
        teamIdList: [],
        statisticsStartTime: "",
        statisticsEndTime: "",
        siteIdList: [],
        userName: "",
        operateType: undefined,
      };
      this.dateRange = [];
    },
    handlePageChange(page) {
      this.form.pageNum = page;
      // 分页加载数据
      this.handleQuery();
    },
    onSizeChange(pageSize) {
      this.form.pageSize = pageSize;
      this.form.pageNum = 1;
      this.handleQuery();
    },
    handleExport() {
      attendanceExport(this.form);
    },
  },
  mounted() {
    this.getProjectList();
  },
};
</script>

<style lang="scss" scoped>
.h-full {
  height: 100%;
}
.border-b {
  border-bottom: 1px solid #e0e0e0;
}
.border-gray-200 {
  border-color: #e0e0e0;
}
.p-6 {
  padding: 1.5rem;
}
.p-16 {
  padding: 4rem;
}
.flex {
  display: flex;
}
.justify-start {
  justify-content: flex-start;
}
.items-center {
  align-items: center;
}
.mb-4 {
  margin-bottom: 1rem;
}
.grid {
  display: grid;
}
.grid-cols-2 {
  grid-template-columns: repeat(2, minmax(0, 1fr));
}
.grid-cols-3 {
  grid-template-columns: repeat(3, minmax(0, 1fr));
}
.grid-cols-4 {
  grid-template-columns: repeat(4, minmax(0, 1fr));
}
.grid-cols-5 {
  grid-template-columns: repeat(5, minmax(0, 1fr));
}
</style>
